import os, sys
import chimera
import getopt

##########################################################################
#set up working directory and import ringer modules
def directorySetup():
	#add library directory to path and get working directory
	path = os.getenv("RINGER")
	working_dir = os.environ.get("PWD")
	#add additional paths to make sure things work on windows machines
	if sys.platform == 'win32':
		path = path.split('\\')
		path.pop(0)
		prefix = "/"+"/".join(path[:(path.index("Cygwin")+1)])
		path = "/"+"/".join(path)
		working_dir = prefix+working_dir
	
	os.chdir(working_dir) 

	return path
##########################################################################
#prints usage statement
def usage():
	print "Correct Usage: \n\tringer -i input_file [-o output_file]"
        sys.exit()
##########################################################################
#check to make sure program is called correctly
def arguments():
	try:
		optlist, args = getopt.getopt(sys.argv[1:], 'i:o:')
	except getopt.GetoptError:
		usage()
	parmin = None
	parmout = None
	for a, i in optlist:
        	if a == '-i':
                	parmin = i
		if a == '-o':
			parmout = i
	if parmin == None:
		usage()
	return parmin, parmout
##########################################################################
#parms parameters from parameter file
def readParameters(parmin, parmout):

	parameters = UserParameters(parmin, parmout)
	parameters.parm_reader()
	return parameters
##########################################################################
#read information for structure file
def structureFile(parameters, path):

	#read templates for moveable sections
	#of residues
	move_set = ringerParms.read_res_templates(path)
	#open pdb structure and create list of residues
	pdb = chimera.openModels.open(parameters.pdbfileName)

	return move_set, pdb
##########################################################################
#read information for structure file
def mapFile(parameters, mapfilename, pdb):
	try: #catch if map is too big
		density = densityMap.open_map(parameters, mapfilename, pdb[0])
	except MemoryError:
		parameters.outfile.write("\nERROR: Memory allocation exceeded " 
				"while reading in %s map." %mapfilename)
		chimera.openModels.close(pdb)
		sys.exit()
	return density
##########################################################################
#set up data structure for residue
def ringerResidue(pdbRes, parameters, move_set):

	#set up data structures for residue
	ringRes = pdbResidues.Residue(pdbRes, parameters.pdbCode)

	#parse residues into chi angles
	ringRes.parse_residue(move_set, pdbRes, parameters)

	#properly process multiply modeled conformers
	tmp_removed = ringRes.removeMulti(parameters)

	return tmp_removed, ringRes
##########################################################################
#create rings and get densities
def generateRings(parameters, ringRes, maps):

	rings,angles = ringMaker.gen_rings(parameters, ringRes.chiList)

	#calculates densities for peak identification
	ringRes.peakList.append(ringMaker.ring_densities(rings,angles,maps[0]))
	if parameters.noisemapfileName:
		ringRes.noiseList.append(ringMaker.
				ring_densities(rings,angles,maps[1]))
##########################################################################
# generate data for chi2 chi1 analysisg
def buildingChi2Chi1(ringRes, parameters, maps):
	if ringRes.chi1 and ringRes.name != 'ALA':
		if len(ringRes.peakList[0].peak1) > 1 \
		    	and len(ringRes.chiList[1]) > 3:
			newChiList = ringRes.createNewChiList(parameters)
			rings,angles = ringMaker.gen_rings(parameters, newChiList)
			
			ringRes.peakList.append(ringMaker.
					ring_densities(rings,angles,maps[0]))
			if parameters.noisemapfileName:
				ringRes.noiseList.append(ringMaker.
					ring_densities(rings,angles,maps[1]))
			#when picking peaks for primary chi1, 
			#turn off both lower and upper cutoffs 
			#for sigma maps or just upper cutoffs for
			#volume maps
			pickPeaks.peak_picking(parameters, ringRes, index=1)

##########################################################################
# write general output to output file
def writeOutput(parameters, 
			removed, badBuiltRes, badPeakRes, 
			rungResidues):
	
	#write out data from removing multiconfs
	pdbResidues.write_removeMulti(parameters, removed)

	#write out problematic built side chains
	pdbResidues.write_badBuiltConformers(parameters, badBuiltRes, badPeakRes)

	#collect lists of peaks by residue type
	peaks_byclass = peakStats.collect_peaks_byclass(rungResidues)
	peakStats.write_tot_stats(parameters, len(rungResidues), peaks_byclass)

	#if requested, write verbose outputs
	if parameters.writePlot:
		verboseOut.write_sigmaplot(parameters, rungResidues, 'signal')
		if parameters.noisemapfileName:
			verboseOut.write_sigmaplot(parameters, rungResidues, 'noise')
	if parameters.writePeakList:
		verboseOut.write_peaklist(parameters, peaks_byclass)
	if parameters.buildChi2Chi1:
		verboseOut.write_chi2chi1(parameters, rungResidues)

##########################################################################
def main():

	parmin, parmout = arguments()
	parameters = readParameters(parmin, parmout)

	#sample chi angles and compute electron density
	parameters.outfile.write("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
	parameters.outfile.write("~~~~~~~~~~~~~~~~~~~~~~~~~~\n")
	parameters.outfile.write("Computing electron density for chi angles...\n\n")

	move_set, pdb = structureFile(parameters, path)
	maps = []
	maps.append(mapFile(parameters, parameters.mapfileName, pdb) )
	if parameters.noisemapfileName:
		maps.append(mapFile(parameters, parameters.noisemapfileName, pdb))

	rungResidues = []
	removed = []
	badBuiltRes = []
	badPeakRes = []
	for pdbRes in pdb[0].residues:
		currRemove, ringRes = ringerResidue(pdbRes, parameters, move_set)
		for rem in currRemove:
			removed.append(rem)
		if len(currRemove) > 1:
			continue
		#generate rings around chi angles and calculates density
		generateRings(parameters, ringRes, maps)

		#identify and collect peaks
		pickPeaks.peak_picking(parameters, ringRes)
		
		#print warning if chi1 is not in same 
		#rotamer well as primary peak 
		if ringRes.chi1 and ringRes.name != 'ALA':
			badChi, badPeak, builtChi = pdbResidues.eval_built_conformer(ringRes)
			if badChi:
				badBuiltRes.append([ringRes, builtChi])
			if badPeak:
				badPeakRes.append(ringRes)
		
		#sample pseudo-chi2 off of secondary chi1
		if parameters.buildChi2Chi1:
			buildingChi2Chi1(ringRes, parameters, maps)
		
		#add residue to list for protein
		rungResidues.append(ringRes)	

	#clean up a bit
	chimera.openModels.close(pdb)
	for density in maps:
		chimera.openModels.close(density)
		
	#write out summary data
	writeOutput(parameters, 
			removed, badBuiltRes, badPeakRes, 
			rungResidues)

	#clean up files
	parameters.close_general_output_file()
###########################################################################
#set up directories for run
path = directorySetup()
sys.path.insert(1, '%s/ringer' %path)

#import ringer-based modules
import ringerParms
from parms import UserParameters
import densityMap
import pdbResidues
import ringMaker
import pickPeaks
import peakStats
import verboseOut

main()


